/*
paging_flush.s - Functions for modifing paging related control register

This file is a part of Fiku.
Copyright (C) 2021 Akib Azmain Turja

Fiku is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Fiku is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

Authors: Akib Azmain Turja
*/

  .section .text
  .global paging_enable_bits_cr0_
  .type paging_enable_bits_cr0_, @function
paging_enable_bits_cr0_:

  // Get parameter
  movl 4(%esp), %ecx

  // Get the current value of cr0
  movl %cr0, %eax

  // Bitwise or with cr0 value
  orl %ecx, %eax

  // Write to cr0
  movl %eax, %cr0

  // Return
  ret
  .size paging_enable_bits_cr0_, . - paging_enable_bits_cr0_

  .global paging_enable_bits_cr4_
  .type paging_enable_bits_cr4_, @function
paging_enable_bits_cr4_:

  // Get parameter
  movl 4(%esp), %ecx

  // Get the current value of cr4
  movl %cr4, %eax

  // Bitwise or with cr4 value
  orl %ecx, %eax

  // Write to cr4
  movl %eax, %cr4

  // Return
  ret
  .size paging_enable_bits_cr4_, . - paging_enable_bits_cr4_

  .global paging_flush_
  .type paging_flush_, @function
paging_flush_:

  // Get the parameter
  movl 4(%esp), %eax

  // Write to cr3
  movl %eax, %cr3

  // Return
  ret
  .size paging_flush_, . - paging_flush_

  .global paging_enable_pae_
  .type paging_enable_pae_, @function
paging_enable_pae_:
  pushl %ebp
  movl %esp, %ebp

  // Backup %ebx
  pushl %ebx

  // Get the parameter
  movl 8(%ebp), %ecx

  // Jump to lower-half before disabling paging, we still have lower-half
  // mapped
  leal 1f, %edx
  subl $0xf0000000, %edx
  jmp *%edx
1:

  // Disable paging
  movl %cr0, %eax
  andl $0x7fffffff, %eax
  movl %eax, %cr0

  // Load %cr3
  movl %ecx, %cr3

  // Modify the control bits
  //
  // WP bit (bit 16 in %cr0): set
  // PAE bit (bit 5 in %cr4): set
  // PG bit (bit 31 in %cr0): set
  movl %cr4, %eax
  orl $0x20, %eax
  movl %eax, %cr4
  movl %cr0, %eax
  orl $0x80010000, %eax
  movl %eax, %cr0

2:

  // Returned to higher-half by the page falt handler, now return from
  // function
  popl %ebx
  popl %ebp
  ret
  .size paging_enable_pae_, . - paging_enable_pae_
