/*
syscall.cpp - System call handlers

This file is a part of Fiku.
Copyright (C) 2021 Akib Azmain Turja

Fiku is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Fiku is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

Authors: Akib Azmain Turja
*/

#include <fiku/syscall.hpp>
#include <fiku/idt.hpp>
#include <fiku/proc.hpp>
#include <fiku/timers/base.hpp>

namespace
{
  enum class status : size_t
    {
      success = 0,
      invalid_syscall = 1,
      invalid_parameter = 2
    };

  using sysfunc_t = void (*) (idt::registers_t &regs);

  void
  sys_thrd_yield (idt::registers_t &regs)
  {
    proc::thrd_yield (proc::get_timer ().read (), &regs);
    regs.eax = static_cast < uint32_t > (status::success);
  }

  sysfunc_t sysfuncs[] =
    {
      /* 0 */ sys_thrd_yield
    };

  void
  dispatcher (uint8_t, uint8_t, idt::registers_t &regs)
  {

    // Syscall number is in %eax, check it
    if (sizeof (sysfuncs) / sizeof (sysfunc_t) < regs.eax)
      {
        regs.eax = static_cast < uint32_t > (status::invalid_syscall);
      }

    // Serve request
    else
      {
        sysfuncs[regs.eax] (regs);
      }
  }
}

namespace syscall
{
  void
  init ()
  {
    idt::set_handler (0xff, dispatcher); // Use interrupt 255 for syscalls
  }
}
