/*
gdt.cpp - GDT related functions for i386 architecture

This file is a part of Fiku.
Copyright (C) 2021 Akib Azmain Turja

Fiku is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Fiku is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

Authors: Akib Azmain Turja
*/

#include <fiku/gdt.hpp>

extern "C" void
gdt_flush__ (void *gdt_ptr);

namespace
{
  struct pointer
  {

    // Size of GDT
    uint16_t limit;

    // Base address of GDT
    uint32_t base;
  }
  __attribute__((packed));

  pointer gdt_ptr;
}

namespace gdt
{
  void
  flush (const entry *table, size_t count)
  {
    gdt_ptr = {
      static_cast < uint16_t > (sizeof (entry) * count - 1),
      reinterpret_cast < uint32_t > (table)
    };
    gdt_flush__ (&gdt_ptr);
  }

  entry *
  init (bool flush)
  {
    static entry gdt[6];

    gdt[1] = entry (0, 0xffffffff, true, true, false, 0x0);
    gdt[2] = entry (0, 0xffffffff, false, true, false, 0x0);
    gdt[3] = entry (0, 0xffffffff, true, true, false, 0x3);
    gdt[4] = entry (0, 0xffffffff, false, true, false, 0x3);

    if (flush)
      {
        gdt::flush (gdt, sizeof (gdt) / sizeof (entry));
      }

    return gdt;
  }

  entry *
  get ()
  {
    return reinterpret_cast < entry *> (gdt_ptr.base);
  }
}
