/*
cpu.cpp - CPU related function

This file is a part of Fiku.
Copyright (C) 2021 Akib Azmain Turja

Fiku is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Fiku is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

Authors: Akib Azmain Turja
*/

#include <fiku/cpu.h>

namespace
{
  cpu_t bsp; // BootStrap Processor or the main processor
}

void
cpu_init ()
{
  bsp.cli_count = 1;
}

cpu_t *
cpu_current ()
{
  return &bsp; // TODO
}

void
cpu_cli ()
{
  if (cpu_current ()->cli_count++ == 0)
    {
      asm volatile ("cli");
    }
}

void
cpu_sti ()
{
  if (--cpu_current ()->cli_count == 0)
    {
      asm volatile ("sti;"
                    "nop;"); // Without it, sti doesn't work on QEMU (BUG).
                             // See <https://stackoverflow.com/q/68135305>.
    }
}

uint64_t
cpu_rdmsr (uint32_t msr)
{
  uint32_t value;
  asm ("rdmsr"
       : "=A" (value)
       : "c" (msr));
  return value;
}

void
cpu_wrmsr (uint64_t data, uint32_t msr)
{
  asm volatile ("wrmsr"
                :
                : "A" (data),
                  "c" (msr));
}
