/*
cpuid.s - Functions for executing CPUID instruction with different commands

This file is a part of Fiku.
Copyright (C) 2021 Akib Azmain Turja

Fiku is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Fiku is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

Authors: Akib Azmain Turja
*/

  .global cpuid_check_eflags_id_bit_
  .type cpuid_check_eflags_id_bit_, @function
cpuid_check_eflags_id_bit_:

  // Push eflags twice, one for modifing and another for comparing
  pushf
  pushf

  // Set the ID bit in the copy of eflags in the stack
  xorl $0x00200000, (%esp)

  // Write to eflags
  popf

  // Push new eflags
  pushf

  // Load new eflags info %eax
  popl %eax

  // Compare %eax with the copy of eflags pushed first
  xorl (%esp), %eax
  andl $0x00200000, %eax
  sarl $21, %eax

  // Restore eflags and return
  popf
  ret
  .size cpuid_check_eflags_id_bit_, . - cpuid_check_eflags_id_bit_

  .global cpuid_exec_p
  .type cpuid_exec_p, @function
cpuid_exec_p:
  pushl %ebp
  movl %esp, %ebp
  pushl %ebx
  pushl %edi

  // Get the function
  movl 8(%ebp), %eax
  movl 12(%ebp), %ebx
  movl 16(%ebp), %ecx
  movl 20(%ebp), %edx

  // Execute cpuid
  cpuid

  // Save return values in memory pointed by second parameter
  movl 24(%ebp), %edi
  movl %eax, (%edi)
  movl %ebx, 4(%edi)
  movl %ecx, 8(%edi)
  movl %edx, 12(%edi)

  // Restore %ebx and %ebp and return
  popl %edi
  popl %ebx
  popl %ebp
  ret
  .size cpuid_exec_p, . - cpuid_exec_p
