/*
idt_routines.s - ISRs for i386 architecture

This file is a part of Fiku.
Copyright (C) 2021 Akib Azmain Turja

Fiku is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Fiku is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

Authors: Akib Azmain Turja
*/

  .section .text
isr_common_asm_:

  // Push general purpose registers
  pushl %ebp
  pushl %eax
  pushl %ebx
  pushl %ecx
  pushl %edx
  pushl %esi
  pushl %edi

  // Push 'ds' register
  movw %ds, %ax
  pushl %eax

  // Load %ebp
  movl %esp, %ebp

  // Load kernel data segment selectors
  movw $0x10, %ax
  movw %ax, %ds
  movw %ax, %es
  movw %ax, %fs
  movw %ax, %gs

  // Call high level C++ handler
  pushl %esp
  call isr_common_
  popl %eax

  .global isr_return_asm_
  .type isr_return_asm_, @function
isr_return_asm_:

  // Restore previous segment selectors
  popl %eax
  movw %ax, %ds
  movw %ax, %es
  movw %ax, %fs
  movw %ax, %gs

  // Restore general purpose registers
  popl %edi
  popl %esi
  popl %edx
  popl %ecx
  popl %ebx
  popl %eax
  popl %ebp

  // Pop interrupt number and error code by changing stack pointer (we
  // can't use pop instruction because all general purpose registers are in
  // use)
  addl $0x8, %esp

  // Return
  iret

  .macro ISR_WITH_CODE n
  .global isr_\n\()__
  .type isr_\n\()__, @function
isr_\n\()__:
  cli
  pushl $\n
  jmp isr_common_asm_
  .endm

  .macro ISR_WITHOUT_CODE n
  .global isr_\n\()__
  .type isr_\n\()__, @function
isr_\n\()__:
  cli
  pushl $0x0
  pushl $\n
  jmp isr_common_asm_
  .endm

ISR_WITHOUT_CODE 0x00
ISR_WITHOUT_CODE 0x01
ISR_WITHOUT_CODE 0x02
ISR_WITHOUT_CODE 0x03
ISR_WITHOUT_CODE 0x04
ISR_WITHOUT_CODE 0x05
ISR_WITHOUT_CODE 0x06
ISR_WITHOUT_CODE 0x07
ISR_WITH_CODE    0x08
ISR_WITHOUT_CODE 0x09
ISR_WITH_CODE    0x0a
ISR_WITH_CODE    0x0b
ISR_WITH_CODE    0x0c
ISR_WITH_CODE    0x0d
ISR_WITH_CODE    0x0e
ISR_WITHOUT_CODE 0x0f
ISR_WITHOUT_CODE 0x10
ISR_WITH_CODE    0x11
ISR_WITHOUT_CODE 0x12
ISR_WITHOUT_CODE 0x13
ISR_WITHOUT_CODE 0x14
ISR_WITH_CODE    0x15
ISR_WITHOUT_CODE 0x16
ISR_WITHOUT_CODE 0x17
ISR_WITHOUT_CODE 0x18
ISR_WITHOUT_CODE 0x19
ISR_WITHOUT_CODE 0x1a
ISR_WITHOUT_CODE 0x1b
ISR_WITHOUT_CODE 0x1c
ISR_WITHOUT_CODE 0x1d
ISR_WITHOUT_CODE 0x1e
ISR_WITHOUT_CODE 0x1f
ISR_WITHOUT_CODE 0x20
ISR_WITHOUT_CODE 0x21
ISR_WITHOUT_CODE 0x22
ISR_WITHOUT_CODE 0x23
ISR_WITHOUT_CODE 0x24
ISR_WITHOUT_CODE 0x25
ISR_WITHOUT_CODE 0x26
ISR_WITHOUT_CODE 0x27
ISR_WITHOUT_CODE 0x28
ISR_WITHOUT_CODE 0x29
ISR_WITHOUT_CODE 0x2a
ISR_WITHOUT_CODE 0x2b
ISR_WITHOUT_CODE 0x2c
ISR_WITHOUT_CODE 0x2d
ISR_WITHOUT_CODE 0x2e
ISR_WITHOUT_CODE 0x2f
ISR_WITHOUT_CODE 0x30
ISR_WITHOUT_CODE 0x31
ISR_WITHOUT_CODE 0x32
ISR_WITHOUT_CODE 0x33
ISR_WITHOUT_CODE 0x34
ISR_WITHOUT_CODE 0x35
ISR_WITHOUT_CODE 0x36
ISR_WITHOUT_CODE 0x37
ISR_WITHOUT_CODE 0x38
ISR_WITHOUT_CODE 0x39
ISR_WITHOUT_CODE 0x3a
ISR_WITHOUT_CODE 0x3b
ISR_WITHOUT_CODE 0x3c
ISR_WITHOUT_CODE 0x3d
ISR_WITHOUT_CODE 0x3e
ISR_WITHOUT_CODE 0x3f
ISR_WITHOUT_CODE 0x40
ISR_WITHOUT_CODE 0x41
ISR_WITHOUT_CODE 0x42
ISR_WITHOUT_CODE 0x43
ISR_WITHOUT_CODE 0x44
ISR_WITHOUT_CODE 0x45
ISR_WITHOUT_CODE 0x46
ISR_WITHOUT_CODE 0x47
ISR_WITHOUT_CODE 0x48
ISR_WITHOUT_CODE 0x49
ISR_WITHOUT_CODE 0x4a
ISR_WITHOUT_CODE 0x4b
ISR_WITHOUT_CODE 0x4c
ISR_WITHOUT_CODE 0x4d
ISR_WITHOUT_CODE 0x4e
ISR_WITHOUT_CODE 0x4f
ISR_WITHOUT_CODE 0x50
ISR_WITHOUT_CODE 0x51
ISR_WITHOUT_CODE 0x52
ISR_WITHOUT_CODE 0x53
ISR_WITHOUT_CODE 0x54
ISR_WITHOUT_CODE 0x55
ISR_WITHOUT_CODE 0x56
ISR_WITHOUT_CODE 0x57
ISR_WITHOUT_CODE 0x58
ISR_WITHOUT_CODE 0x59
ISR_WITHOUT_CODE 0x5a
ISR_WITHOUT_CODE 0x5b
ISR_WITHOUT_CODE 0x5c
ISR_WITHOUT_CODE 0x5d
ISR_WITHOUT_CODE 0x5e
ISR_WITHOUT_CODE 0x5f
ISR_WITHOUT_CODE 0x60
ISR_WITHOUT_CODE 0x61
ISR_WITHOUT_CODE 0x62
ISR_WITHOUT_CODE 0x63
ISR_WITHOUT_CODE 0x64
ISR_WITHOUT_CODE 0x65
ISR_WITHOUT_CODE 0x66
ISR_WITHOUT_CODE 0x67
ISR_WITHOUT_CODE 0x68
ISR_WITHOUT_CODE 0x69
ISR_WITHOUT_CODE 0x6a
ISR_WITHOUT_CODE 0x6b
ISR_WITHOUT_CODE 0x6c
ISR_WITHOUT_CODE 0x6d
ISR_WITHOUT_CODE 0x6e
ISR_WITHOUT_CODE 0x6f
ISR_WITHOUT_CODE 0x70
ISR_WITHOUT_CODE 0x71
ISR_WITHOUT_CODE 0x72
ISR_WITHOUT_CODE 0x73
ISR_WITHOUT_CODE 0x74
ISR_WITHOUT_CODE 0x75
ISR_WITHOUT_CODE 0x76
ISR_WITHOUT_CODE 0x77
ISR_WITHOUT_CODE 0x78
ISR_WITHOUT_CODE 0x79
ISR_WITHOUT_CODE 0x7a
ISR_WITHOUT_CODE 0x7b
ISR_WITHOUT_CODE 0x7c
ISR_WITHOUT_CODE 0x7d
ISR_WITHOUT_CODE 0x7e
ISR_WITHOUT_CODE 0x7f
ISR_WITHOUT_CODE 0x80
ISR_WITHOUT_CODE 0x81
ISR_WITHOUT_CODE 0x82
ISR_WITHOUT_CODE 0x83
ISR_WITHOUT_CODE 0x84
ISR_WITHOUT_CODE 0x85
ISR_WITHOUT_CODE 0x86
ISR_WITHOUT_CODE 0x87
ISR_WITHOUT_CODE 0x88
ISR_WITHOUT_CODE 0x89
ISR_WITHOUT_CODE 0x8a
ISR_WITHOUT_CODE 0x8b
ISR_WITHOUT_CODE 0x8c
ISR_WITHOUT_CODE 0x8d
ISR_WITHOUT_CODE 0x8e
ISR_WITHOUT_CODE 0x8f
ISR_WITHOUT_CODE 0x90
ISR_WITHOUT_CODE 0x91
ISR_WITHOUT_CODE 0x92
ISR_WITHOUT_CODE 0x93
ISR_WITHOUT_CODE 0x94
ISR_WITHOUT_CODE 0x95
ISR_WITHOUT_CODE 0x96
ISR_WITHOUT_CODE 0x97
ISR_WITHOUT_CODE 0x98
ISR_WITHOUT_CODE 0x99
ISR_WITHOUT_CODE 0x9a
ISR_WITHOUT_CODE 0x9b
ISR_WITHOUT_CODE 0x9c
ISR_WITHOUT_CODE 0x9d
ISR_WITHOUT_CODE 0x9e
ISR_WITHOUT_CODE 0x9f
ISR_WITHOUT_CODE 0xa0
ISR_WITHOUT_CODE 0xa1
ISR_WITHOUT_CODE 0xa2
ISR_WITHOUT_CODE 0xa3
ISR_WITHOUT_CODE 0xa4
ISR_WITHOUT_CODE 0xa5
ISR_WITHOUT_CODE 0xa6
ISR_WITHOUT_CODE 0xa7
ISR_WITHOUT_CODE 0xa8
ISR_WITHOUT_CODE 0xa9
ISR_WITHOUT_CODE 0xaa
ISR_WITHOUT_CODE 0xab
ISR_WITHOUT_CODE 0xac
ISR_WITHOUT_CODE 0xad
ISR_WITHOUT_CODE 0xae
ISR_WITHOUT_CODE 0xaf
ISR_WITHOUT_CODE 0xb0
ISR_WITHOUT_CODE 0xb1
ISR_WITHOUT_CODE 0xb2
ISR_WITHOUT_CODE 0xb3
ISR_WITHOUT_CODE 0xb4
ISR_WITHOUT_CODE 0xb5
ISR_WITHOUT_CODE 0xb6
ISR_WITHOUT_CODE 0xb7
ISR_WITHOUT_CODE 0xb8
ISR_WITHOUT_CODE 0xb9
ISR_WITHOUT_CODE 0xba
ISR_WITHOUT_CODE 0xbb
ISR_WITHOUT_CODE 0xbc
ISR_WITHOUT_CODE 0xbd
ISR_WITHOUT_CODE 0xbe
ISR_WITHOUT_CODE 0xbf
ISR_WITHOUT_CODE 0xc0
ISR_WITHOUT_CODE 0xc1
ISR_WITHOUT_CODE 0xc2
ISR_WITHOUT_CODE 0xc3
ISR_WITHOUT_CODE 0xc4
ISR_WITHOUT_CODE 0xc5
ISR_WITHOUT_CODE 0xc6
ISR_WITHOUT_CODE 0xc7
ISR_WITHOUT_CODE 0xc8
ISR_WITHOUT_CODE 0xc9
ISR_WITHOUT_CODE 0xca
ISR_WITHOUT_CODE 0xcb
ISR_WITHOUT_CODE 0xcc
ISR_WITHOUT_CODE 0xcd
ISR_WITHOUT_CODE 0xce
ISR_WITHOUT_CODE 0xcf
ISR_WITHOUT_CODE 0xd0
ISR_WITHOUT_CODE 0xd1
ISR_WITHOUT_CODE 0xd2
ISR_WITHOUT_CODE 0xd3
ISR_WITHOUT_CODE 0xd4
ISR_WITHOUT_CODE 0xd5
ISR_WITHOUT_CODE 0xd6
ISR_WITHOUT_CODE 0xd7
ISR_WITHOUT_CODE 0xd8
ISR_WITHOUT_CODE 0xd9
ISR_WITHOUT_CODE 0xda
ISR_WITHOUT_CODE 0xdb
ISR_WITHOUT_CODE 0xdc
ISR_WITHOUT_CODE 0xdd
ISR_WITHOUT_CODE 0xde
ISR_WITHOUT_CODE 0xdf
ISR_WITHOUT_CODE 0xe0
ISR_WITHOUT_CODE 0xe1
ISR_WITHOUT_CODE 0xe2
ISR_WITHOUT_CODE 0xe3
ISR_WITHOUT_CODE 0xe4
ISR_WITHOUT_CODE 0xe5
ISR_WITHOUT_CODE 0xe6
ISR_WITHOUT_CODE 0xe7
ISR_WITHOUT_CODE 0xe8
ISR_WITHOUT_CODE 0xe9
ISR_WITHOUT_CODE 0xea
ISR_WITHOUT_CODE 0xeb
ISR_WITHOUT_CODE 0xec
ISR_WITHOUT_CODE 0xed
ISR_WITHOUT_CODE 0xee
ISR_WITHOUT_CODE 0xef
ISR_WITHOUT_CODE 0xf0
ISR_WITHOUT_CODE 0xf1
ISR_WITHOUT_CODE 0xf2
ISR_WITHOUT_CODE 0xf3
ISR_WITHOUT_CODE 0xf4
ISR_WITHOUT_CODE 0xf5
ISR_WITHOUT_CODE 0xf6
ISR_WITHOUT_CODE 0xf7
ISR_WITHOUT_CODE 0xf8
ISR_WITHOUT_CODE 0xf9
ISR_WITHOUT_CODE 0xfa
ISR_WITHOUT_CODE 0xfb
ISR_WITHOUT_CODE 0xfc
ISR_WITHOUT_CODE 0xfd
ISR_WITHOUT_CODE 0xfe
ISR_WITHOUT_CODE 0xff
