/*
cpu.h - CPU related function

This file is a part of Fiku.
Copyright (C) 2021 Akib Azmain Turja

Fiku is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Fiku is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

Authors: Akib Azmain Turja
*/

#pragma once

#include <stddef.h>
#include <stdint.h>

#ifdef __cplusplus
extern "C"
{
#endif

typedef struct
{
  size_t cli_count;
} cpu_t;

// Initializes
void
cpu_init ();

// Returns the CPU executing caller
cpu_t *
cpu_current ();

// Stops interrupts
void
cpu_cli ();

// Starts interrupts
void
cpu_sti ();

// Reads from model specific register
uint64_t
cpu_rdmsr (uint32_t msr);

// Writes to model specific register
void
cpu_wrmsr (uint64_t data, uint32_t msr);

// Freezes the CPU
__attribute__ ((noreturn)) void
cpu_freeze ();

#ifdef __cplusplus
}
#endif
