/*
cpuid.h - Functions for executing CPUID functions

This file is a part of Fiku.
Copyright (C) 2021 Akib Azmain Turja

Fiku is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Fiku is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

Authors: Akib Azmain Turja
*/

#pragma once

#include <stdint.h>

#ifdef __cplusplus
extern "C"
{
#endif

// Checks whether CPUID instruction is available
bool
cpuid_check ();

// Check whether extended functions are available
bool
cpuid_check_extended ();

struct cpuid_data_t
{
  uint32_t eax;
  uint32_t ebx;
  uint32_t ecx;
  uint32_t edx;
};

// Executes CPUID and puts return value to ret
void
cpuid_exec_p (cpuid_data_t function, cpuid_data_t* ret);

// Executes CPUID
cpuid_data_t
cpuid_exec (cpuid_data_t function);

// Returns vendor string returned by CPUID
const char*
cpuid_vendor ();

// Returns CPU features
cpuid_data_t
cpuid_getfeatures ();

// Returns whether the CPU supports Page Size Extension (aka PSE)
bool
cpuid_feature_pse ();

// Returns whether the CPU support Physical Address Extension (aka PAE)
bool
cpuid_feature_pae ();

// Returns the maximum physical address width supported by the CPU
uint8_t
cpuid_pae_maxphyaddr ();

// Returns whether the CPU supports execute disable (NX) bit
bool
cpuid_feature_nx ();

// Returns whether the CPU supports MSR (Model Specific Register)
bool
cpuid_feature_msr ();

#ifdef __cplusplus
}
#endif
