/*
base.cpp - Base class for timers

This file is a part of Fiku.
Copyright (C) 2021 Akib Azmain Turja

Fiku is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Fiku is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

Authors: Akib Azmain Turja
*/

#pragma once

#include <stddef.h>
#include <fiku/idt.hpp>

namespace timer
{
  class basic_timer
  {
  public:
    basic_timer () = default;

    virtual
    ~basic_timer () = default;

    // Handler type
    using handler_t = void (*) (uint8_t interrupt_number, size_t time,
                                idt::registers_t &registers);

    // Set the timer to call handler every interval millisecond (pass 0 to
    // only measure time, but not call the handler)
    virtual void
    set (size_t interval, handler_t handler = nullptr) = 0;

    // Start the timer
    virtual void
    start () = 0;

    // Reads the time elasped since the handler was called, or if the
    // handler has never been called, the time elasped since the timer
    // started
    virtual size_t
    read () const = 0;

    // Stop the timer
    virtual void
    stop () = 0;
  };
}
