/*
new.hpp - Definitions for operator new

This file is a part of Fiku.
Copyright (C) 2021 Akib Azmain Turja

Fiku is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Fiku is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

Authors: Akib Azmain Turja
*/

#pragma once

#include <stddef.h>

namespace gstd
{
  enum class align_val_t : size_t
    {};

  struct nothrow_t
  {
    explicit nothrow_t () = default;
  };

  extern const nothrow_t nothrow;

  using new_handler = void (*) ();

  new_handler
  get_new_handler () noexcept;

  new_handler
  set_new_handler (new_handler handler) noexcept;

  /* Not sure what to do with the following variables
  static constexpr size_t hardware_destructive_interference_size =
    alignof (max_align_t);

  static constexpr size_t hardware_constructive_interference_size =
    alignof (max_align_t);
  */
}

void *
operator new (size_t size);

void *
operator new (size_t size, gstd::align_val_t alignment);

void *
operator new (size_t size, const gstd::nothrow_t &) noexcept;

void *
operator new (size_t size, gstd::align_val_t alignment,
              const gstd::nothrow_t &) noexcept;

void *
operator new (size_t size, void *ptr) noexcept;

void *
operator new[] (size_t size);

void *
operator new[] (size_t size, gstd::align_val_t alignment);

void *
operator new[] (size_t size, const gstd::nothrow_t &) noexcept;

void *
operator new[] (size_t size, gstd::align_val_t alignment,
                const gstd::nothrow_t &) noexcept;

void *
operator new[] (size_t size, void *ptr) noexcept;

void
operator delete (void *ptr) noexcept;

void
operator delete (void *ptr, size_t size) noexcept;

void
operator delete (void *ptr, gstd::align_val_t alignment) noexcept;

void
operator delete (void *ptr, size_t size,
                 gstd::align_val_t alignment) noexcept;

void
operator delete (void *ptr, gstd::nothrow_t &) noexcept;

void
operator delete (void *ptr, gstd::align_val_t alignment,
                 gstd::nothrow_t &) noexcept;

void
operator delete (void *ptr, void *) noexcept;

void
operator delete[] (void *ptr) noexcept;

void
operator delete[] (void *ptr, size_t size) noexcept;

void
operator delete[] (void *ptr, gstd::align_val_t alignment) noexcept;

void
operator delete[] (void *ptr, size_t size,
                   gstd::align_val_t alignment) noexcept;

void
operator delete[] (void *ptr, gstd::nothrow_t &) noexcept;

void
operator delete[] (void *ptr, gstd::align_val_t alignment,
                   gstd::nothrow_t &) noexcept;

void
operator delete[] (void *ptr, void *) noexcept;
