/*
string.h - Null-terminated byte string related C functions

This file is a part of Fiku.
Copyright (C) 2021 Akib Azmain Turja

Fiku is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Fiku is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

Authors: Akib Azmain Turja
*/

#pragma once

#include <stddef.h>

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * @brief Fills buffer with given character
 *
 * @details @rst
 *
 * This function fill ``count`` bytes of given buffer ``dest`` with given
 * character ``ch``.
 *
 * .. note::
 *
 *      Before writing to buffer, ``ch`` is converted to ``unsigned char``.
 *
 * @endrst
 *
 * @param dest destination buffer
 * @param ch fill byte
 * @param count count of bytes to write
 * @return ``dest``
 */
void *memset (void *dest, int ch, size_t count);

/**
 * @brief Copies data from one buffer to another
 *
 * @details @rst
 *
 * This function copies ``count`` bytes of data from source buffer ``src`` to
 * destination buffer ``dest``.
 *
 * @endrst
 *
 * @param dest destination buffer
 * @param src source buffer
 * @param count count of bytes to copy
 * @return ``dest``
 */
void *memcpy (void *dest, const void *src, size_t count);

/**
 * @brief Moves data from one buffer to another
 *
 * @details @rst
 *
 * This function moves ``count`` bytes of data from source buffer ``src`` to
 * destination buffer ``dest``. The given buffers can overlap.
 *
 * @endrst
 *
 * @param dest destination buffer
 * @param src source buffer
 * @param count count of bytes to copy
 * @return ``dest``
 */
void *memmove (void *dest, const void *src, size_t count);

/**
 * @brief Returns the size of given string without the null character
 */
size_t strlen (const char *str);

#ifdef __cplusplus
}
#endif
