/*
threads.h - C Thread support library

This file is a part of Fiku.
Copyright (C) 2021 Akib Azmain Turja

Fiku is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Fiku is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

Authors: Akib Azmain Turja
*/

#pragma once

#include <stddef.h>

#ifdef __cplusplus
extern "C"
{
#endif

typedef struct _thrd_info *thrd_t;

// Possible return values of functions
enum
  {
    thrd_success = 0,
    thrd_nomem = 1,
    thrd_busy = 2,
    thrd_timedout = 3,
    thrd_error = 4
  };

typedef int (*thrd_start_t) (void *arg);

// Creates a thread
int
thrd_create (thrd_t *thrdp, thrd_start_t func, void *arg);

// Compare two threads
int
thrd_equal (thrd_t lhs, thrd_t rhs);

// Returns current thread
thrd_t
thrd_current ();

// Yields current timeslice
void
thrd_yield ();

// Exits current thread
__attribute__ ((noreturn)) void
thrd_exit (int retval);

// Blocks until the thread has exited
int
thrd_join (thrd_t thrd, int *retptr);

// Returns a pointer to the low level thread structure
void *
thrd_handle (thrd_t thrd);

typedef struct
{
  volatile bool self_locked;
  volatile bool locked;
  thrd_t locker;
  bool recursive;
  size_t recursion_count;
} mtx_t;

enum
  {
    mtx_plain = 0,
    mtx_recursive = 1
  };

// Initializes a mutex
int
mtx_init (mtx_t *mtx, int type);

// Locks a mutex, blocks until success
int
mtx_lock (mtx_t *mtx);

// Tries to lock a mutex, returns on failure
int
mtx_trylock (mtx_t *mtx);

// Unlocks a mutex
int
mtx_unlock (mtx_t *mtx);

// Destroyes a mutex
int
mtx_destroy (mtx_t *mtx);

#ifdef __cplusplus
}
#endif
