/*
stdlib.cpp - Standard utility C function implementations

This file is a part of Fiku.
Copyright (C) 2021 Akib Azmain Turja

Fiku is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Fiku is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

Authors: Akib Azmain Turaj
*/

#include <stdlib.h>
#include <limits.h>

namespace
{
  template < class type > type
  array_to (const char *str)
  {

    // Is the number negative? By default, NO
    bool negative = false;

    // Check whether the sign is specified
    if (*str == '-')
      {
        negative = true;
        str++;
      }
    else if (*str == '+')
      {
        str++;
      }

    // atoi: Array TO Integer!
    type num = 0;
    while (*str >= '0' && *str <= '9')
      {
        num = num * 10 + *(str++) - '0';
      }

    // Change the sign of number if required and return
    return negative ? -num : num;
  }
}

#ifdef LIBC_ABS
int
abs (int x)
{
  return x < 0 ? -x : x;
}
#endif

#ifdef LIBC_LABS
long
labs (long x)
{
  return x < 0 ? -x : x;
}
#endif

#ifdef LIBC_ATOI
int
atoi (const char *str)
{
  return array_to < int > (str);
}
#endif

#ifdef LIBC_ATOL
long
atol (const char *str)
{
  return array_to < long > (str);
}
#endif

#ifdef LIBC_ATOLL
long long
atoll (const char *str)
{
  return array_to < long long > (str);
}
#endif
