/*
threads.cpp - C Thread support library

This file is a part of Fiku.
Copyright (C) 2021 Akib Azmain Turja

Fiku is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Fiku is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

Authors: Akib Azmain Turja
*/

#include <threads.h>
#include <fiku/proc.hpp>

struct _thrd_info
{
  proc::thrd_t handle;
  static constexpr size_t stack_size = 0x1000;
  char stack[stack_size];
  volatile bool exited;
  volatile int retval;
};

int
thrd_equal (thrd_t lhs, thrd_t rhs)
{
  return lhs == rhs;
}

thrd_t
thrd_current ()
{
  return reinterpret_cast < thrd_t > (proc::thrd_current ());
}

__attribute__ ((noreturn)) void
thrd_exit (int retval)
{
  thrd_t thrd = thrd_current ();

  thrd->retval = retval;
  thrd->exited = true;

  thrd->handle.thrd_state = thrd->handle.thrd_killed;
  proc::thrd_rm (&thrd->handle);
  thrd_yield ();

  while (true);
}

int
thrd_join (thrd_t thrd, int *retptr)
{
  while (!thrd->exited)
    {
      thrd_yield ();
    }

  if (retptr)
    {
      *retptr = thrd->retval;
    }

  return thrd_success;
}

void *
thrd_handle (thrd_t thrd)
{
  return &thrd->handle;
}

int
mtx_init (mtx_t *mtx, int type)
{
  mtx->self_locked = false;
  mtx->locked = false;
  mtx->locker = nullptr;
  mtx->recursive = type & mtx_recursive;
  mtx->recursion_count = 0;
  return thrd_success;
}

int
mtx_destroy (mtx_t *)
{
  // Nothing to do...
  return thrd_success;
}
