/*
linker.ld - Linker script for linking Fiku

This file is a part of Fiku.
Copyright (C) 2021 Akib Azmain Turja

Fiku is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Fiku is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

Authors: Akib Azmain Turja
*/

/* Start execution from symbol "_start" */
ENTRY (_start)

SECTIONS
{
  . = 0xf0100000;

  .text ALIGN (4K) : AT (ADDR (.text) - 0xf0000000)
  {
    text_begin_ = .;
    *(.multiboot2)
    *(.text)
    *(.text.*)
    text_end_ = .;
  }

  .init ALIGN (4K) : AT (ADDR (.init) - 0xf0000000)
  {
    init_begin_ = .;
    *crti.o(.init)
    *crtbegin.o(.init)
    *(EXCLUDE_FILE (*crti.o *crtn.o *crtbegin.o *crtend.o) .init)
    *crtend.o(.init)
    *crtn.o(.init)
    init_end_ = .;
  }

  .fini ALIGN (4K) : AT (ADDR (.fini) - 0xf0000000)
  {
    fini_begin_ = .;
    *crti.o(.fini)
    *crtbegin.o(.fini)
    *(EXCLUDE_FILE (*crti.o *crtn.o *crtbegin.o *crtend.o) .fini)
    *crtend.o(.fini)
    *crtn.o(.fini)
    fini_end_ = .;
  }

  .rodata ALIGN (4K) : AT (ADDR (.rodata) - 0xf0000000)
  {
    rodata_begin_ = .;
    *(.rodata)
    *(.rodata.*)
    rodata_end_ = .;
  }

  .ctors ALIGN (4K) : AT (ADDR (.ctors) - 0xf0000000)
  {
    ctors_begin_ = .;
    *crti.o(.ctors)
    *crtbegin.o(.ctors)
    *(EXCLUDE_FILE (*crti.o *crtn.o *crtbegin.o *crtend.o) .ctors)
    *(SORT (EXCLUDE_FILE (*crti.o *crtn.o *crtbegin.o *crtend.o) .ctors.*))
    *crtend.o(.ctors)
    *crtn.o(.ctors)
    ctors_end_ = .;
  }

  .dtors ALIGN (4K) : AT (ADDR (.dtors) - 0xf0000000)
  {
    dtors_begin_ = .;
    *crti.o(.dtors)
    *crtbegin.o(.dtors)
    *(SORT (EXCLUDE_FILE (*crti.o *crtn.o *crtbegin.o *crtend.o) .dtors.*))
    *(EXCLUDE_FILE (*crti.o *crtn.o *crtbegin.o *crtend.o) .dtors)
    *crtend.o(.dtors)
    *crtn.o(.dtors)
    dtors_end_ = .;
  }

  .data ALIGN (4K) : AT (ADDR (.data) - 0xf0000000)
  {
    data_begin_ = .;
    *(.data)
    data_end_ = .;
  }

  .bss ALIGN (4K) : AT (ADDR (.bss) - 0xf0000000)
  {
    bss_begin_ = .;
    *(COMMON)
    *(.bss)
    bss_end_ = .;
  }
}
